"""
Exceptions for SmartyStreets requests.
"""


class SmartyStreetsError(Exception):
    """Unknown SmartyStreets error"""

    def __str__(self):
        return self.__doc__


class SmartyStreetsInputError(SmartyStreetsError):
    """HTTP 400 Bad input. Required fields missing from input or are malformed."""


class SmartyStreetsAuthError(SmartyStreetsError):
    """HTTP 401 Unauthorized. Authentication failure; invalid credentials"""


class SmartyStreetsPaymentError(SmartyStreetsError):
    """HTTP 402 Payment required. No active subscription found."""


class SmartyStreetsServerError(SmartyStreetsError):
    """HTTP 500 Internal server error. General service failure; retry request."""


ERROR_CODES = {
    400: SmartyStreetsInputError,
    401: SmartyStreetsAuthError,
    402: SmartyStreetsPaymentError,
    500: SmartyStreetsServerError,
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Client module for connecting to and interacting with SmartyStreets API
"""

import json
import requests

from .data import Address, AddressCollection
from .exceptions import SmartyStreetsError, ERROR_CODES


def validate_args(f):
    """
    Ensures that *args consist of a consistent type

    :param f: any client method with *args parameter
    :return: function f
    """
    def wrapper(self, args):
        arg_types = set([type(arg) for arg in args])
        if len(arg_types) > 1:
            raise TypeError("Mixed input types are not allowed")
        elif list(arg_types)[0] not in (dict, str):
            raise TypeError("Only dict and str types accepted")
        return f(self, args)
    return wrapper


def truncate_args(f):
    """
    Ensures that *args do not exceed a set limit or are truncated to meet that limit

    :param f: any Client method with *args parameter
    :return: function f
    """
    def wrapper(self, args):
        if len(args) > 100:
            if self.truncate_addresses:
                args = args[:100]
            else:
                raise ValueError("This exceeds 100 address at a time SmartyStreets limit")
        return f(self, args)
    return wrapper


class Client(object):
    """
    Client class for interacting with the SmartyStreets API
    """
    BASE_URL = "https://api.smartystreets.com/"

    def __init__(self, auth_id, auth_token, standardize=False, invalid=False, logging=True,
                 accept_keypair=False, truncate_addresses=False):
        """
        Constructs the client

        :param auth_id: authentication ID from SmartyStreets
        :param auth_token: authentication token
        :param standardize: boolean include addresses that match zip+4 in addition to DPV confirmed
                addresses
        :param invalid: boolean to include address candidates that may not be deliverable
        :param logging: boolean to allow SmartyStreets to log requests
        :param accept_keypair: boolean to toggle default keypair behavior
        :param truncate_addresses: boolean to silently truncate address lists in excess of the
                SmartyStreets maximum rather than raise an error.
        :return: the configured client object
        """
        self.auth_id = auth_id
        self.auth_token = auth_token
        self.standardize = standardize
        self.invalid = invalid
        self.logging = logging
        self.accept_keypair = accept_keypair
        self.truncate_addresses = truncate_addresses

    def post(self, endpoint, data):
        """
        Executes the HTTP POST request

        :param endpoint: string indicating the URL component to call
        :param data: the data to submit
        :return: the dumped JSON response content
        """
        headers = {
            'Content-Type': 'application/json',
            'Accept': 'application/json',
            'x-standardize-only': 'true' if self.standardize else 'false',
            'x-include-invalid': 'true' if self.invalid else 'false',
            'x-accept-keypair': 'true' if self.accept_keypair else 'false',
        }
        if not self.logging:
            headers['x-suppress-logging'] = 'false'

        params = {'auth-id': self.auth_id, 'auth-token': self.auth_token}
        url = self.BASE_URL + endpoint
        response = requests.post(url, json.dumps(data), params=params, headers=headers)
        if response.status_code == 200:
            return response.json()
        raise ERROR_CODES.get(response.status_code, SmartyStreetsError)

    @truncate_args
    @validate_args
    def street_addresses(self, addresses):
        """
        API method for verifying street address and geolocating

        Returns an AddressCollection always for consistency. In common usage it'd be simple and
        sane to return an Address when only one address was searched, however this makes
        populating search addresses from lists of unknown length problematic. If that list
        returns only one address now the code has to check the type of return value to ensure
        that it isn't applying behavior for an expected list type rather than a single dictionary.

        >>> client.street_addresses(["100 Main St, Anywhere, USA"], ["6 S Blvd, Richmond, VA"])
        >>> client.street_addresses([{"street": "100 Main St, anywhere USA"}, ... ])

        :param addresses: 1 or more addresses in string or dict format
        :return: an AddressCollection
        """

        # While it's okay in theory to accept freeform addresses they do need to be submitted in
        # a dictionary format.
        if type(addresses[0]) != dict:
            addresses = [{'street': arg for arg in addresses}]

        return AddressCollection(self.post('street-address', data=addresses))

    def street_address(self, address):
        """
        Geocode one and only address, get a single Address object back

        >>> client.street_address("100 Main St, Anywhere, USA")
        >>> client.street_address({"street": "100 Main St, anywhere USA"})

        :param address: string or dictionary with street address information
        :return: an Address object or None for no match
        """
        address = self.street_addresses([address])
        if not len(address):
            return None
        return Address(address[0])

    def zipcode(self, *args):
        raise NotImplementedError("You cannot lookup zipcodes yet")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()


with open("smartystreets/__init__.py", "r") as module_file:
    for line in module_file:
        if line.startswith("__version__"):
            version_string = line.split("=")[1]
            version = version_string.strip().replace("'", "")


requirements = [
    'requests>=2.0.0',
]

test_requirements = [
    'mock>=1.0.0',
    'responses==0.3.0',
]

setup(
    name='smartystreets.py',
    version=version,
    description='A wrapper for the SmartyStreets address validation and geolocation API"',
    long_description=readme + '\n\n' + history,
    author='Ben Lopatin',
    author_email='ben@wellfire.co',
    url='https://github.com/bennylope/smartystreets.py',
    packages=[
        'smartystreets',
    ],
    package_dir={'smartystreets':
                 'smartystreets'},
    include_package_data=True,
    install_requires=requirements,
    license="BSD",
    zip_safe=False,
    keywords='smarty streets',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    test_suite='tests',
    test_requirements=test_requirements,
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_smartystreets
----------------------------------

Tests for `smartystreets` module.

Note that for now async behavior is provided via grequests, which uses gevent, which
in packaged form is only supported on Python 2.7. As such this functionality is not
expected to work on Python 3 (for now) which means the tests here are skipped for
Python 3. This is a sad.
"""
import responses
import sys
import unittest
from mock import MagicMock

from smartystreets.data import Address, AddressCollection
from smartystreets.exceptions import (SmartyStreetsInputError, SmartyStreetsAuthError,
                                      SmartyStreetsPaymentError, SmartyStreetsServerError)

PY_VERSION = int(sys.version[0])

try:
    import grequests
except ImportError:
    # This satisfies the named uses below without needing to import
    # each and every time the AsyncClient is used.
    from smartystreets import Client as AsyncClient
else:
    from smartystreets.async import AsyncClient


@unittest.skipIf(PY_VERSION > 2, "No gevent support in Python 3")
class TestAsyncClient(unittest.TestCase):

    def setUp(self):
        self.client = AsyncClient(auth_id='blah', auth_token='blibbidy')

    @responses.activate
    def test_input_error(self):
        responses.add(responses.POST, 'https://api.smartystreets.com/street-address',
                      body='', status=400,
                      content_type='application/json')
        self.assertRaises(SmartyStreetsInputError, self.client.street_addresses, [{}, {}])

    @responses.activate
    def test_auth_error(self):
        responses.add(responses.POST, 'https://api.smartystreets.com/street-address',
                      body='', status=401,
                      content_type='application/json')
        self.assertRaises(SmartyStreetsAuthError, self.client.street_addresses, [{}, {}])

    @responses.activate
    def test_payment_error(self):
        responses.add(responses.POST, 'https://api.smartystreets.com/street-address',
                      body='', status=402,
                      content_type='application/json')
        self.assertRaises(SmartyStreetsPaymentError, self.client.street_addresses, [{}, {}])

    @responses.activate
    def test_server_error(self):
        responses.add(responses.POST, 'https://api.smartystreets.com/street-address',
                      body='', status=500,
                      content_type='application/json')
        self.assertRaises(SmartyStreetsServerError, self.client.street_addresses, [{}, {}])

    @responses.activate
    def test_addresses_response(self):
        """Ensure address return an AddressCollection"""
        responses.add(responses.POST, 'https://api.smartystreets.com/street-address',
                      body='[{"street_address": "100 Main St"}, {"street_address": "200 Main St"}]',
                      status=200, content_type='application/json')
        response = self.client.street_addresses([{"street": "100 Main st"},
                                                 {"street": "200 Main St"}])
        self.assertIsInstance(response[0], AddressCollection)
        self.assertEqual(2, len(response[0]))
